--[[
	Player for game
	Caveware Digital :: 2017
]]


local Player = {}


--[[
	Creates the player at a position
]]
function Player:init(x, y, params)
	-- Set position
	self._x = x
	self._y = y

	-- Set size
	self._w = 16
	self._h = 16

	-- Personal qualities
	self._facing_dir = "down"
	self._mode = "parasite"

	__FLOWER_TIMER = 0
end


function Player:update(dt)
	if (self:isTransforming()) then
		local transform = self._transformation
		transform.distance = transform.distance + dt / 0.25

		if (transform.distance >= 1) then
			self:setMode(transform.next_mode)
			if (self._transformation.after) then
				self._transformation.after()
			end
			self._transformation = nil
		end
	else
		self:manageMovement(dt)

		-- If can, update movement lock
		local state = StateManager:getActiveState()
		if (self._move_lock) then
			local speed = self:getWalkingSpeed()
			self._move_lock.distance = self._move_lock.distance + dt / speed

			-- If the cycle is completed, set position and remove lock
			if (self._move_lock.distance >= 1) then
				self._x = self._move_lock.x
				self._y = self._move_lock.y
				self._move_lock = nil
			end
		elseif (state.current_map == maps.cell_a2) then
			if (self._x == 80 and self._y == 96) then
				__FLOWER_TIMER = __FLOWER_TIMER + dt
			end

			if (__FLOWER_TIMER >= 10) then
				Dialogue:set({
					"You fall into some kind of hidden passage.",
					function ()
						__LAST_MODE = self:getMode()
						self:startTransformation("empty", function ()
							bgm.block_a:stop()
							state:setMap(maps.cave, "door_cave")
							bgm.cave:start()
							Player:setMode(__LAST_MODE)
						end)
					end
				})
			end
		else
			__FLOWER_TIMER = 0
		end


		local usable = self:getUsable()
		self._usable_exists = usable and true or false
		if (usable and InputManager:wasPressed("action")) then
			self._move_lock = nil
			usable:onUse()
		end
	end
end


function Player:draw()
	-- Determine position to draw in based on move lock if exists
	local x, y = self:getDrawPosition()

	if (self:isTransforming()) then
		local nx, ny = self:getDrawPosition(self._transformation.next_mode)

		local distance = self._transformation.distance
		local steps = {
			"left",
			"up",
			"right",
			"down"
		}
		local image_step = "idle_" .. steps[math.floor(distance * 16) % 4 + 1]
		local alpha1 = math.max(1 - distance * 2, 0)
		local alpha2 = math.max(distance * 2 - 1, 0)

		love.graphics.setColor(255 * alpha1, 255 * alpha1, 255 * alpha1, 255 * (1 - alpha2))
		local image = gfx.char[self._mode][image_step]
		love.graphics.draw(image, x, y)

		love.graphics.setColor(255 * alpha2, 255 * alpha2, 255 * alpha2, 255 * (1 - alpha1))
		local nimage = gfx.char[self._transformation.next_mode][image_step]
		love.graphics.draw(nimage, nx, ny)

		love.graphics.setColor(255, 255, 255)
	else
		local image = gfx.char[self._mode][self:getImage()]
		love.graphics.draw(image, x, y)

		-- Draw prompt if usable item exists
		if (self._usable_exists) then
			love.graphics.draw(gfx.misc.prompt, x + 10, y - 4)
		end
	end
end



function Player:getDrawPosition(mode)
	local x, y = self._x, self._y
	if (self._move_lock) then
		x = x + (self._move_lock.x - x) * self._move_lock.distance
		y = y + (self._move_lock.y - y) * self._move_lock.distance
	end
	y = y + self:getImageOffset(mode)

	return x, y
end


--[[
	If there is a usable item in front of the player, return it. Otherwise nil.
	OUTPUT: The thing
]]
function Player:getUsable()
	local player_box = Player:getCollisionBox()
	for key, portal in ipairs(StateManager:getActiveState().portals) do
		if (Hit:checkRectangles(player_box, portal) and not portal.prop.arrive_only) then
			return {
				onUse = function ()
					-- Allowed if no class
					local allowed = true
					if (portal.prop.for_class) then
						allowed = false
						local classes = string.split(portal.prop.for_class, " ")
						for key, value in ipairs(classes) do
							if (value == self._mode) then
								allowed = true
							end
						end
					end

					if (allowed) then
						local active_state = StateManager:getActiveState()
						local segments = string.split(portal.prop.to, " ")
						if (segments[1] == "warden_office") then
							if (Flag:get("warden__open")) then
								StateManager:push("cutscene", cutscenes.warden)
							else
								Dialogue:set({"The door to the warden's office is locked."})
							end
						elseif (segments[1] == "cell_b2") then
							if (Flag:get("cs-colin__step-1")) then
								Dialogue:set({"You don't want to go back in there."})
							else
								StateManager:push("cutscene", cutscenes.colin)
							end
						else
							active_state:loadMap(maps[segments[1]], segments[2])
						end
					else
						Dialogue:set({"This door is locked. Only the guard can open this."})
					end
				end
			}
		end
	end

	-- Figure out direction
	local horizontal, vertical = 0, 0
	if (self._facing_dir == "right") then
		horizontal = horizontal + 1
	elseif (self._facing_dir == "left") then
		horizontal = horizontal - 1
	elseif (self._facing_dir == "down") then
		vertical = vertical + 1
	elseif (self._facing_dir == "up") then
		vertical = vertical - 1
	end

	-- Check NPCs
	player_box.x = player_box.x + horizontal * GRID_SIZE
	player_box.y = player_box.y + vertical * GRID_SIZE

	for key, npc in ipairs(StateManager:getActiveState().npcs) do
		if (Hit:checkRectangles(player_box, npc:getCollisionBox()) and
		    npc:isTalkable()) then
			return npc
		end
	end

	-- If nothing can be found, gracefully nope the fuck out of there
	return nil
end


function Player:getCollisionBox()
	return {
		x = self._x,
		y = self._y,
		w = self._w,
		h = self._h
	}
end


--[[
	Which image should be drawn? We decide here!
	OUTPUT: Name of image to draw
]]
function Player:getImage()
	local direction = self:getMovementDirection()
	if (direction == "idle") then
		return "idle_" .. self._facing_dir
	else
		local frames = self:getWalkingFrames() - 1
		local number = math.ceil(self._move_lock.distance * frames + 0.5)
		return "walk_" .. direction .. "_" .. number
	end
end


--[[
	Which image should be drawn? We decide here!
	OUTPUT: Name of image to draw
]]
function Player:getImageOffset(mode)
	local mode = mode or self._mode
	if (mode == "parasite") then
		return 0
	end
	return -16
end


function Player:getMode()
	return self._mode
end


--[[
	Returns the direction in which the player is -moving-
	OUTPUT: Movement direction or idle
]]
function Player:getMovementDirection()
	local lock = self._move_lock
	if (not lock) then
		return "idle"
	elseif (lock.x < self._x) then
		return "left"
	elseif (lock.x > self._x) then
		return "right"
	elseif (lock.y < self._y) then
		return "up"
	elseif (lock.y > self._y) then
		return "down"
	else
		return "idle"
	end
end


--[[
	Returns how many frames are in the walking animation for a mode
	OUTPUT: Frame count
]]
function Player:getWalkingFrames()
	if (self._mode == "guard_2" or self._mode == "prisoner_3" or self._mode == "prisoner_8" or self._mode == "prisoner_4") then
		local direction = Player:getMovementDirection()
		if (direction == "left" or direction == "right") then
			return 2
		end
		return 3
	elseif (self._mode == "parasite") then
		return 2
	elseif (self._mode == "scientist") then
		return 3
	elseif (self._mode == "scientist_jail") then
		return 3
	end
end


--[[
	Returns how fast player can move based on mode
	OUTPUT: How much time in secs it will take the player to reach next square
]]
function Player:getWalkingSpeed()
	if (self._mode == "guard_2") then
		return 0.24
	elseif (self._mode == "parasite") then
		return 0.3
	elseif (self._mode == "scientist") then
		return 0.18
	elseif (self._mode == "prisoner_3") then
		return 0.19
	elseif (self._mode == "prisoner_4") then
		return 0.16
	elseif (self._mode == "prisoner_8") then
		return 0.14
	end

	return 0.3
end


function Player:isTransforming()
	return self._transformation and true or false
end


--[[
	Actually checks if the player should move
	INPUT: Delta timer
]]
function Player:manageMovement(dt)
	-- How much the player will move horizontally or vertically
	local horizontal, vertical;

	-- Calculate movement
	horizontal = math.sign(InputManager:get("right") - InputManager:get("left"))
	vertical = 0
	if (horizontal == 0) then
		vertical = math.sign(InputManager:get("down") - InputManager:get("up"))
	end

	-- Actually move the character if possible
	if (not self._move_lock and (horizontal + vertical) ~= 0) then
		local new_x, new_y =
			self._x + GRID_SIZE * horizontal,
			self._y + GRID_SIZE * vertical

		self._move_lock = {
			distance = 0,
			x = new_x,
			y = new_y
		}

		-- Update facing dir if not idle
		self._facing_dir = Player:getMovementDirection()

		if (not self:pointIsClear(new_x, new_y)) then
			self._move_lock = nil
		end
	end
end


function Player:pointIsClear(x, y)
	local player_box = {
		x = x,
		y = y,
		w = self._w,
		h = self._h
	}

	-- Check for NPCs blocking the way
	for key, npc in ipairs(StateManager:getActiveState().npcs) do
		if (Hit:checkRectangles(player_box, npc:getCollisionBox())) then
			return false
		end
	end

	-- Check for NPCs blocking the way
	for key, object in ipairs(StateManager:getActiveState().collisions) do
		if (Hit:checkRectangles(player_box, object)) then
			return false
		end
	end

	return true
end


function Player:setMode(new_mode)
	self._mode = new_mode
end


function Player:startTransformation(new_mode, after)
	self._transformation = {
		distance = 0,
		next_mode = new_mode,
		after = after
	}
end



return Player
