--[[
	Play state
	Caveware Digital :: 2017
]]


local current_folder = ...
local PlayState = GameState:new()


--[[
	Initialises the play state
	INPUT: options - Default settings for this state
]]
function PlayState:start(options)
	Player:init(64, 64)
	self.transition = {
		distance = 0.51,
		done = true
	}
	self:loadMap(maps.block_a, "door_holding_down")
end


--[[
	Performed upon this state being popped
]]
function PlayState:stop()
	-- Kill everything about this state
end


--[[
	Update each loop
	INPUT: dt - Delta time
]]
function PlayState:update(dt)
	if (self.transition) then
		self.transition.distance = self.transition.distance + dt / 0.45

		if (self.transition.distance >= 0.5 and not self.transition.done) then
			self:setMap(self.transition.next_map, self.transition.next_checkpoint)
			self.transition.done = true
		end

		if (self.transition.distance >= 1) then
			self.transition = nil
		end
	else
		Player:update(dt)
		for key, npc in ipairs(self.npcs) do
			npc:update(dt)
		end

		if (InputManager:wasPressed("y")) then
			EssenceMenu:open()
		end
	end
end


--[[
	Draws
]]
function PlayState:draw()
	-- Translate screen to focus on player
	local px, py = Player:getDrawPosition()
	if (Player:isTransforming()) then
		local dy = Player:getImageOffset(Player._transformation.next_mode) - Player:getImageOffset()
		local py_diff = dy * Player._transformation.distance
		py = py + py_diff
	end
	px = px + Player._w / 2
	py = py + Player._h / 2
	local tx, ty = (Flags.GAME_WIDTH / 2) - px, (Flags.GAME_HEIGHT / 2) - py
	love.graphics.translate(math.round(tx), math.round(ty))

	-- Map must be drawn
	MapTool.draw(self.current_map)

	-- Sort drawables by y value
	local drawables = newExtendedTable({})
	drawables:insert(Player)
	drawables:insertSingles(self.npcs)
	drawables:sort(function (a, b)
		return a._y < b._y
	end)

	-- Draw all drawables
	for key, drawable in ipairs(drawables) do
		drawable:draw()
	end
	MapTool.drawFG(self.current_map)

	love.graphics.translate(-math.round(tx), -math.round(ty))

	if (self.transition) then
		local alpha = math.sin(math.rad(self.transition.distance * 180))
		love.graphics.setColor(0, 0, 0, alpha * 255)
		love.graphics.rectangle("fill", 0, 0, Flags.GAME_WIDTH, Flags.GAME_HEIGHT)
	end
end




function PlayState:loadMap(map_to_load, checkpoint)
	bgm.cave:stop()

	if (map_to_load == maps.yard) then
		bgm.block_a:stop()
		bgm.block_b:stop()
		bgm.cafe:stop()
		bgm.yard:play()
	elseif (self.current_map == maps.yard) then
		bgm.yard:stop()

		if (map_to_load == maps.block_a) then
			bgm.block_a:play()
		elseif (map_to_load == maps.block_b) then
			bgm.block_b:play()
		elseif (map_to_load == maps.cafe) then
			bgm.cafe:play()
		end
	end

	if (not self.current_map) then
		self:setMap(map_to_load, checkpoint)
	else
		self.transition = {
			distance = 0,
			done = false,
			next_map = map_to_load,
			next_checkpoint = checkpoint
		}
	end
end


function PlayState:setMap(map_to_load, checkpoint)
	local checkpoint = checkpoint or "spawn"
	self.collisions = {}
	self.current_map = map_to_load
	self.npcs = newExtendedTable({})
	self.portals = newExtendedTable({})

	for key, layer in ipairs(map_to_load.layers) do
		if (layer.name == "collisions") then
			self.collisions = layer.objects
		elseif (layer.name == "entities") then
			for key, entity in ipairs(layer.objects) do
				local class = NPCs[entity.name]
				self.npcs:insert(class:new(entity.x, entity.y))
			end
		elseif (layer.name == "portals") then
			for key, portal in ipairs(layer.objects) do
				if (portal.name == checkpoint) then
					Player._x = portal.x
					Player._y = portal.y
				end

				-- Add all non-portals to the table
				if (portal.name ~= "spawn") then
					self.portals:insert(portal)
				end
			end
		end
	end
end


return PlayState
