--[[
	Manages different kinds of inputs
	Caveware Digital :: 2017
]]


local InputManager = {
	_controls = require("controls"),
	_deadzone = 0.15,
	_previous_values = {},
	_values = {}
}


--[[
	Performed on input manager load
]]
function InputManager:load()
	self:_checkInputs()
end


--[[
	Performed on input manager update
]]
function InputManager:update()
	self:_checkInputs()
end


--[[
	Returns numerical value of input, restricted by deadzone
	INPUT: Key name
	OUTPUT: Number between 0 and 1
]]
function InputManager:get(key)
	local value = self._values[key]
	return (self._values[key] > self._deadzone) and value or 0
end


--[[
	Returns true if the control key is currently being held
	INPUT: Control key to check
	OUTPUT: true / false (Boolean)
]]
function InputManager:isHeld(key)
	return self._values[key] > self._deadzone
end


--[[
	Returns true if the control key was pressed since the last update
	INPUT: Control key to check
	OUTPUT: true / false (Boolean)
]]
function InputManager:wasPressed(key)
	return self._values[key] > self._deadzone and
	       self._previous_values[key] <= self._deadzone
end


function InputManager:claimWasPressed(key)
	self._previous_values[key] = self._values[key]
end


--[[
	Returns true if the control key was released since the last update
	INPUT: Control key to check
	OUTPUT: true / false (Boolean)
]]
function InputManager:wasReleased(key)
	return self._values[key] <= self._deadzone and
	       self._previous_values[key] > self._deadzone
end


--[[
	Updates the tables which track control values
]]
function InputManager:_checkInputs()
	-- Iterate through all control tables
	for control_key, control in pairs(self._controls) do
		-- Assume by default that the control is not active
		local value = 0

		-- Iterate through all commands in the specific control
		for command_key, command in ipairs(control) do
			-- Split the command so that it can be interpreted
			local command_segments = command:split("_")

			-- Depending on control type, call appropriate function
			local control_value = (command_segments[1] == "key") and
				 self:_interpretKey(command_segments[2]) or
				 self:_interpretPad(command_segments)

			-- Set value to control_value if larger than current value
			value = math.max(value, control_value)
		end

		-- Update previous value for control and set new one
		self._previous_values[control_key] = self._values[control_key] or 0
		self._values[control_key] = value
	end
end


--[[
	Returns 1 or 0 depending on whether a key is held
	INPUT: Key to check
	OUTPUT: 1 or 0, indicating a integral conversion of the boolean (Number)
]]
function InputManager:_interpretKey(key)
	return love.keyboard.isDown(key) and 1 or 0
end


--[[
	Returns an analogue response based on a pad control's state
	INPUT: Segments of control
	OUTPUT: A float between 0 and 1 (Number)
]]
function InputManager:_interpretPad(segments)
	local joystick = love.joystick.getJoysticks()[1]

	if (not joystick or not joystick:isGamepad()) then
		return 0
	end

	if (segments[2] == "button") then
		return joystick:isGamepadDown(segments[3]) and 1 or 0
	elseif (segments[2] == "dpad") then
		local hat_direction = joystick:getHat(1)

		for i = 1, #hat_direction do
			if (segments[3] == hat_direction:sub(i, i)) then
				return 1
			end
		end

		return 0
	elseif (segments[2] == "stick") then
		local neg = (segments[4] == "neg") and -1 or 1
		return math.max(joystick:getGamepadAxis(segments[3]) * neg, 0)
	end

	return 0
end


return InputManager
