--[[
 	Manages scaling of the screen
	Caveware Digital :: 2017
]]


local ScaleManager = {}


--[[
	Initialises the scaling manager
]]
function ScaleManager:load()
	self._canvas = love.graphics.newCanvas(Flags.GAME_WIDTH, Flags.GAME_HEIGHT)
	self._scale_mode = Flags.GAME_DEFAULT_SCALE_MODE
end


--[[
	Checks to see where a position on screen is in-game, and if it is in_game
	OUTPUT: Position table with x, y, and in_game. Also a valid Hit rectangle.
]]
function ScaleManager:getPositionInGame(x, y)
	local s = self:_getScaleVariables()

	local canvas = {
		x = 0,
		y = 0,
		w = self._canvas:getWidth(),
		h = self._canvas:getHeight()
	}

	local position = {
		x = (x - s.x) / s.scale_w,
		y = (y - s.y) / s.scale_h,
		w = 1,
		h = 1
	}

	position.in_game = Hit:checkRectangles(position, canvas)

	return position
end


--[[
	Used to determine current scaling mode
	OUTPUT: Current scaling mode
]]
function ScaleManager:getScaleMode()
	return self._scale_mode
end


--[[
	Used to set new scaling mode
	INPUT: New scaling mode
]]
function ScaleManager:setScaleMode(new_scale_mode)
	self._scale_mode = new_scale_mode
end


--[[
	Draws a function inside the scalable canvas
	INPUT: Function to draw
]]
function ScaleManager:draw(fn)
	-- Use fn to draw to canvas
	love.graphics.setCanvas(self._canvas)
		love.graphics.clear()
		fn()
	love.graphics.setCanvas()

	love.graphics.setColor(255, 255, 255)

	-- Default for none
	local s = self:_getScaleVariables()
	love.graphics.draw(self._canvas, s.x, s.y, 0, s.scale_w, s.scale_h)
end


--[[
	Returns essential variables for scaling function
	OUTPUT: Table of scaling properties
]]
function ScaleManager:_getScaleVariables()
	local mode = self:getScaleMode()
	local s = {
		scale_w = 1,
		scale_h = 1,
		x = 0,
		y = 0
	}

	local width, height = love.graphics.getWidth(), love.graphics.getHeight()

	if (mode == "full") then
		s.scale_w = width / Flags.GAME_WIDTH
		s.scale_h = height / Flags.GAME_HEIGHT
	elseif (mode == "aspect_ratio") then
		local scale_w = width / Flags.GAME_WIDTH
		local scale_h = height / Flags.GAME_HEIGHT

		s.x = math.max((scale_w - scale_h) * Flags.GAME_WIDTH / 2, 0)
		s.y = math.max((scale_h - scale_w) * Flags.GAME_HEIGHT / 2, 0)

		local scale_min = math.min(scale_w, scale_h)
		s.scale_w, s.scale_h = scale_min, scale_min
	end

	return s
end


return ScaleManager